//
// Created by trotFunky on 03/11/2019.
//

#include <Engine.h>
#include "DataHandling/Texture.h"
#include "MovementManager.h"
#include "TileTypes.h"

static Texture map;
static constexpr float horizontal_speed = 0.05;
static constexpr int units_per_pixel = 2;
static constexpr int map_side = 32;
static constexpr int corner_pos = (map_side*units_per_pixel);

void manage_inputs()
{
    if (InputStatus::is_key_pressed(0x1B))
    {
        glutDestroyWindow(glutGetWindow());
        exit(EXIT_SUCCESS);
    }

    Vec3d translation{0,0,0};

    // TODO : camera_translate function in MovementManager that handles both the correct translation and rollback.
    if (InputStatus::is_special_key_pressed(GLUT_KEY_RIGHT))
    {
        translation.z += horizontal_speed;
    }
    if (InputStatus::is_special_key_pressed(GLUT_KEY_LEFT))
    {
        translation.z -= horizontal_speed;
    }

    if (InputStatus::is_special_key_pressed(GLUT_KEY_UP))
    {
        translation.x += horizontal_speed;
    }
    if (InputStatus::is_special_key_pressed(GLUT_KEY_DOWN))
    {
        translation.x -= horizontal_speed;
    }

    constrained_translation(translation,OGLE::camera,map,units_per_pixel);

    Vec3i map_colour = current_map_colour(OGLE::camera.get_eyepos(), map, units_per_pixel);
    if (map_colour == end)
    {
        std::cout << "Success !" << std::endl;
        glutDestroyWindow(glutGetWindow());
        exit(EXIT_SUCCESS);
    }

    Vec2i mouse_delta = InputStatus::get_mouse_delta(true);
    if (mouse_delta.x != 0 || mouse_delta.y != 0)
    {
        OGLE::camera.rotate({0,
                             -mouse_delta.x / InputStatus::mouse_sensitivity.x,
                             -mouse_delta.y / InputStatus::mouse_sensitivity.y});
    }
}

void display()
{
    manage_inputs();

    glEnable(GL_ALPHA_TEST);

    glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
    glBindTexture(GL_TEXTURE_2D,map.opengl_id[0]);

    // Deactivate interpolation
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

    glColor4f(1.0f, 1.0f, 1.0f,1.00);

    glBegin(GL_QUADS);
    glTexCoord2f(0,0);
    glVertex3i(0,0,0);

    glTexCoord2d(1,0);
    glVertex3i(0,0,corner_pos);

    glTexCoord2d(1,1);
    glVertex3i(corner_pos,0,corner_pos);

    glTexCoord2d(0,1);
    glVertex3i(corner_pos,0,0);
    glEnd();

    glDisable(GL_ALPHA_TEST);

}

int main(int argc, char** argv)
{
    OGLE::setup(argc,argv,display,"The Labyrinth !");

    map.load_rgb_tga("resources/Map.tga");

    OGLE::camera.set_position({corner_pos/2,1,corner_pos/2});

    glutMainLoop();
    return 0;
}